@@ -3,7 +3,7 @@ use 5.006001;
 use strict;
 use warnings;
 
-our $VERSION = '0.16';
+our $VERSION = '0.17';
 
 use IO::All;
 use YAML::XS;
@@ -19,6 +19,7 @@ field ext => '';
 field help => 0;
 field clean => 0;
 field compile => 0;
+field sample => 0;
 field run => 0;
 field html => 0;
 field start => 0;
@@ -30,6 +31,12 @@ field config => {
     width => 80,
     list_indent => 10,
     skip => 0,
+    vim => 'vim',
+    # gvim options
+    fuopt => 'maxhorz,maxvert',
+    guioptions => 'egmLtT',
+    guicursor => 'a:blinkon0-ver25-Cursor',
+    guifont => 'Bitstream_Vera_Sans_Mono:h18',
 };
 
 sub new {
@@ -40,11 +47,15 @@ sub usage {
     return <<'...';
     Usage: vroom [options]
 
+    -new        - Create a sample 'slides.vroom' file
     -vroom      - Start slideshow
-    -compile    - Generate files
+    -compile    - Generate slides
+    -html       - Publish slides as HTML
+
+    -skip=#     - Skip # of slides
+    -input=name - Specify an input file name
+
     -clean      - Delete generated files
-    -run        - Run a slide file (perl or yaml)
-    -html       - Publish as HTML
     -help       - Get help!
 ...
 }
@@ -54,7 +65,10 @@ sub vroom {
 
     $self->getOptions;
 
-    if ($self->run) {
+    if ($self->sample) {
+        $self->sampleSlides;
+    }
+    elsif ($self->run) {
         $self->runSlide;
     }
     elsif ($self->clean) {
@@ -90,6 +104,7 @@ Create a new directory for your slides and run vroom from there.
 
     GetOptions(
         "help" => \$self->{help},
+        "new" => \$self->{sample},
         "clean" => \$self->{clean},
         "compile" => \$self->{compile},
         "run" => \$self->{run},
@@ -106,8 +121,10 @@ Create a new directory for your slides and run vroom from there.
 sub cleanUp {
     my $self = shift;
     unlink(glob "0*");
+    unlink('.help');
     unlink('.vimrc');
-    unlink('run.yaml');
+    unlink('.gvimrc');
+    unlink('run.slide');
 }
 
 sub cleanAll {
@@ -118,21 +135,45 @@ sub cleanAll {
 
 sub runSlide {
     my $self = shift;
-    my $slide = shift @ARGV;
-    if ($slide =~ /\.pl$/) {
-        exec "perl $slide";
-    }
-    if ($slide =~ /\.yaml/) {
-        my $yaml < io($slide);
-        $yaml =~ s/^\s*\n//;
-        $yaml =~ s/\n\s*$/\n/;
-        if ($yaml =~ /^    \S/) {
-            $yaml =~ s/^    //mg;
-        }
-        $yaml > io('run.yaml');
+    my $slide = $ARGV[0];
 
-        exec "perl -MYAML::XS -MData::Dumper -e '\$Data::Dumper::Terse = 1; \$Data::Dumper::Indent = 1; print Dumper YAML::XS::LoadFile(shift)' run.yaml";
+    if ($slide =~ /\.pl$/) {
+        exec "clear; $^X $slide";
     }
+
+    $self->trim_slide;
+
+    if ($slide =~ /\.py$/) {
+        exec "clear; python run.slide";
+    }
+    elsif ($slide =~ /\.rb$/) {
+        exec "clear; ruby run.slide";
+    }
+    elsif ($slide =~ /\.php$/) {
+        exec "clear; php run.slide";
+    }
+    elsif ($slide =~ /\.js$/) {
+        exec "clear; js run.slide";
+    }
+    elsif ($slide =~ /\.hs$/) {
+        exec "clear; runghc run.slide";
+    }
+    elsif ($slide =~ /\.yaml$/) {
+        exec "clear; $^X -MYAML::XS -MData::Dumper -e '\$Data::Dumper::Terse = 1; \$Data::Dumper::Indent = 1; print Dumper YAML::XS::LoadFile(shift)' run.slide";
+    }
+}
+
+sub trim_slide {
+    my $self = shift;
+    my $slide = $ARGV[0];
+
+    my $text < io($slide);
+    $text =~ s/^\s*\n//;
+    $text =~ s/\n\s*$/\n/;
+    while ($text !~ /^\S/m) {
+        $text =~ s/^ //mg;
+    }
+    $text > io('run.slide');
 }
 
 sub makeSlides {
@@ -141,6 +182,7 @@ sub makeSlides {
     $self->getInput;
     $self->buildSlides;
     $self->writeVimrc;
+    $self->writeHelp;
 }
 
 sub makeHTML {
@@ -195,8 +237,29 @@ sub indexTemplate {
 <head>
 <title>[% config.title | html %]</title>
 <meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
+<script>
+function navigate(e) {
+    var keynum = (window.event) // IE
+        ? e.keyCode
+        : e.which;
+    if (keynum == 13 || keynum == 32) {
+        window.location = "001.html";
+        return false;
+    }
+    return true;
+}
+</script>
+<style>
+body {
+    font-family: sans-serif;
+}
+h4 {
+    color: #888;
+}
+</style>
 </head>
 <body>
+<h4>Use SPACEBAR to peruse the slides or click one to start...<h4>
 <h1>[% config.title | html %]</h1>
 <ul>
 [% FOR entry = index -%]
@@ -254,14 +317,21 @@ function navigate(e) {
 
 sub getInput {
     my $self = shift;
-    my $stream = io($self->input)->all
+    my @stream = io($self->input)->slurp
         or croak "No input provided. Make a file called 'slides.vroom'";
+    my $stream = join '', map {
+        /^----\s+include\s+(\S+)/
+        ? scalar(io($1)->all)
+        : $_
+    } @stream;
     $self->stream($stream);
 }
 
 sub buildSlides {
     my $self = shift;
-    my @split = grep length, split /^(----\ *.*)\n/m, $self->stream;
+    my @split = split /^(----\ *.*)\n/m, $self->stream;
+    shift @split;
+    @split = grep length, @split;
     push @split, '----' if $split[0] =~ /\n/;
     my (@raw_configs, @raw_slides);
     while (@split) {
@@ -312,6 +382,11 @@ sub buildSlides {
             if ($slide =~ s/^\ *!(.*\n)//m) {
                 $slide .= $1;
             }
+            # this option can't be applied ahead of time
+            if ($config->{undent}) {
+                my $undent = $config->{undent};
+                $slide =~ s/^.{$undent}//gm;
+            }
             $slide =~ s{^\ *==\ +(.*?)\ *$}
                        {' ' x (($self->config->{width} - length($1)) / 2) . $1}gem;
             my $suf = $suffix++;
@@ -332,19 +407,36 @@ sub formatNumber {
     return sprintf "%0${digits}d", $number;
 }
 
+my $types = {
+    perl => 'pl', pl => 'pl',
+    ruby => 'rb', rb => 'rb',
+    python => 'py', py => 'py',
+    haskell => 'hs', hs => 'hs',
+    javascript => 'js', js => 'js',
+    actionscript => 'as', as => 'as',
+    shell => 'sh', sh => 'sh',
+    php => 'php',
+    java => 'java',
+    yaml => 'yaml',
+    xml => 'xml',
+    json => 'json',
+    html => 'html',
+    make => 'make',
+    diff => 'diff',
+    conf => 'conf',
+};
 sub parseSlideConfig {
     my $self = shift;
     my $string = shift;
     my $config = {};
+    my $type_list = join '|', keys %$types;
     for my $option (split /\s*,\s*/, $string) {
         $config->{$1} = 1
-            if $option =~ /^(
-                config|skip|center|replace|
-                perl|ruby|python|js|
-                yaml|make|html
-            )$/x;
+            if $option =~ /^(config|skip|center|replace|$type_list)$/;
         $config->{indent} = $1
             if $option =~ /i(\d+)/;
+        $config->{undent} = $1
+            if $option =~ /i-(\d+)/;
     }
     return $config;
 }
@@ -372,8 +464,7 @@ sub applyOptions {
                    {$1 . ' ' x (($self->config->{width} - length($2)) / 2) . $2}gem;
         $slide =~ s{^\s*$}{}gm;
     }
-
-    if (defined $config->{indent}) {
+    elsif (defined $config->{indent}) {
         my $indent = $config->{indent};
         $slide =~ s{^(\+?)}{$1 . ' ' x $indent}gem;
     }
@@ -382,16 +473,13 @@ sub applyOptions {
         $slide =~ s{^(\+?)}{$1 . ' ' x $indent}gem;
     }
 
-    my $ext = 
-        $config->{perl} ? ".pl" :
-        $config->{js} ? ".js" :
-        $config->{python} ? ".py" :
-        $config->{ruby} ? ".rb" :
-        $config->{html} ? ".html" :
-        $config->{shell} ? ".sh" :
-        $config->{yaml} ? ".yaml" :
-        $config->{make} ? ".mk" :
-        "";
+    my $ext = '';
+    for my $key (keys %$config) {
+        if (my $e = $types->{$key}) {
+            $ext = ".$e";
+            last;
+        }
+    }
     $self->ext($ext);
 
     return $slide;
@@ -421,8 +509,8 @@ sub padFullScreen {
 sub writeVimrc {
     my $self = shift;
 
-    my $home_vroom = File::HomeDir->my_home . "/.vroom/vimrc";
-    my $home_vimrc = -e $home_vroom ? io($home_vroom)->all : ''; 
+    my $home_vimrc = File::HomeDir->my_home . "/.vroom/vimrc";
+    my $home_vimrc_content = -e $home_vimrc ? io($home_vimrc)->all : ''; 
 
     die <<'...'
 The .vimrc in your current directory does not look like vroom created it.
@@ -433,27 +521,156 @@ time, and rerun vroom. You should not get this message again.
 ...
     if -e '.vimrc' and io('.vimrc')->getline !~ /Vroom-\d\.\d\d/;
 
-    my $title = "%f         " . $self->config->{title};
-    $title =~ s/\s/_/g;
+    my $title = "%-20f " . $self->config->{title};
+    $title =~ s/\s/\\ /g;
     io(".vimrc")->print(<<"...");
 " This .vimrc file was created by Vroom-$VERSION
 map <SPACE> :n<CR>:<CR>gg
 map <BACKSPACE> :N<CR>:<CR>gg
 map R :!vroom -run %<CR>
-map Q :q!<CR>
-map O :!open <cWORD><CR><CR>
-map E :e <cWORD><CR>
-map ! G:!open <cWORD><CR><CR>
+map RR :!vroom -run %<CR>
+map VV :!vroom -vroom<CR>
+map QQ :q!<CR>
+map OO :!open <cWORD><CR><CR>
+map EE :e <cWORD><CR>
+map !! G:!open <cWORD><CR><CR>
+map ?? :e .help<CR>
 set laststatus=2
 set statusline=$title
 
-" Overrides from $home_vroom
-$home_vimrc
+" Overrides from $home_vimrc
+$home_vimrc_content
+...
+
+    if ($self->config->{vim} =~ /\bgvim\b/) {
+        my $home_gvimrc = File::HomeDir->my_home . "/.vroom/gvimrc";
+        my $home_gvimrc_content = -e $home_gvimrc ? io($home_gvimrc)->all : ''; 
+
+        io(".gvimrc")->print(<<"...");
+set fuopt=${\ $self->config->{fuopt}}
+set guioptions=${\ $self->config->{guioptions}}
+set guifont=${\ $self->config->{guifont}}
+set guicursor=${\ $self->config->{guicursor}}
+
+" Overrides from $home_gvimrc
+$home_gvimrc_content
+...
+    }
+}
+
+sub writeHelp {
+    my $self = shift;
+    io('.help')->print(<<'...');
+
+    <SPACE>         Advance
+    <BACKSPACE>     Go back
+
+    ??              Help
+    QQ              Quit Vroom
+
+    RR              Run slide as a program
+    VV              vroom --vroom 
+    EE              Edit file under cursor
+    OO              Open file under cursor (Mac OS X)
+
+
+    (Press SPACE to leave Help screen and continue)
+
 ...
 }
 
 sub startUp {
-    exec "vim 0*";
+    my $self = shift;
+    my $vim = $self->config->{vim};
+    exec "$vim 0*";
+}
+
+sub sampleSlides {
+    my $self = shift;
+    my $file = $self->input;
+    die <<"..." if -e $file;
+'$file' already exists.
+
+If you really want to generate a new template slides file,
+please delete or move this one.
+...
+    io($file)->print(<<'...');
+# This is a sample Vroom input file. It should help you get started.
+#
+# Edit this file with your content. Then run `vroom --vroom` to start
+# the show!
+#
+# See `perldoc Vroom::Vroom` for complete details.
+#
+---- config
+# Basic config options.
+title: Vroom!
+indent: 5
+height: 18
+width: 69
+skip: 0
+
+# The following options are for Gvim usage.
+# vim: gvim
+# fuopt: maxhorz,maxvert
+# guioptions: egmLtT
+# guicursor: a:blinkon0-ver25-Cursor
+# guifont: Bitstream_Vera_Sans_Mono:h18
+
+---- center
+Vroom!
+
+by Ingy döt Net
+
+(hint: press the spacebar)
+
+----
+== Slideshows in Vim
+
+* Hate using PowerPoint or HTML Slides for Talks?
++* Use Vroom!
+
++* You can write you slides in Vim...
+* ...and present them in Vim!
+
+----
+== Getting Started
+
+* Write a file called 'slides.vroom'.
+  * Do this in a new directory.
+* Run 'vroom --vroom'.
+* Voilà!
+
+----
+== Navigation
+
+* Hit <SPACE> to move forward.
+* Hit <BACKSPACE> to go backwards.
+* Hit 'Q' to quit.
+
+---- perl,i4
+# This is some Perl code.
+# Notice the syntax highlighting.
+# Run it with the <RR> vim command.
+for my $word (qw(Vroom totally rocks!)) {
+    print "$word\n";
+}
+
+----
+== Get Vroom!
+
+* http://search.cpan.org/dist/Vroom/
+* http://github.com/ingydotnet/vroom-pm/
+
+----
+== Vroom as HTML
+
+* http://ingydotnet.github.com/vroom-pm/
+
+----
+== The End
+...
+    print "'$file' created.\n";
 }
 
 =encoding utf8
@@ -466,13 +683,14 @@ Vroom::Vroom - Slide Shows in Vim
 
     > mkdir MySlides    # Make a Directory for Your Slides
     > cd MySlides       # Go In There
-    > vim slides.vroom  # Write Some Slides
+    > vroom -new        # Create Example Slides File
+    > vim slides.vroom  # Edit the File and Add Your Own Slides
     > vroom --vroom     # Show Your Slides
-    > vroom --html      # Publis Your Slides as HTML
+    > vroom --html      # Publish Your Slides as HTML
 
 =head1 DESCRIPTION
 
-Ever given a Slide Show and needed to switch over to the shell?
+Ever given a Slide Show and needed to switch over to Vim?
 
 Now you don't ever have to switch again. You're already there.
 
@@ -481,10 +699,11 @@ style, much like Spork and Sporx do. The difference is that your slides
 don't compile to HTML or JavaScript or XUL. They get turned into a set
 of files that begin with '0', like '03' or '07c' or '05b.pl'.
 
-The slides are named in alpha order. That means you can bring them all
-into a Vim session with the command: C<vim 0*>. C<vroom --vroom> does
-exactly that.
+The slides are named in alphabetic order. That means you can bring them
+all into a Vim session with the command: C<vim 0*>. C<vroom --vroom>
+does exactly that.
 
+You can do things like advance to the next slide with the spacebar.
 Vroom creates a file called C<./.vimrc> with helpful key mappings for
 navigating a slideshow. See L<KEY MAPPINGS> below.
 
@@ -504,17 +723,43 @@ Vroom has a few command line options:
 
 =over
 
-=item vroom
+=item vroom --new
 
-Just running vroom will compiles 'slides.vroom' into slide files.
+Write an example C<slides.vroom> file. This example contains all the
+config options and also examples of all the Vroom syntax features.
 
 =item vroom --vroom
 
-Compile and start vim show.
+Compile (create) the slides files from the input file and start vim
+show.
+
+=item vroom --compile
+
+Just compile the slides.
+
+=item vroom --html
+
+Publish the slides to HTML, with embedded JavaScript to navigate with
+the spacebar and backspace keys. Created in the C<html/> subdirectory.
 
 =item vroom --clean
 
-Clean up all the compiled  output files.
+Clean up all the compiled output files.
+
+=item vroom <action> --skip=#
+
+The skip option takes a number as its input and skips that number of
+files during compilation. This is useful when you are polishing your slides
+and are finished with the first 50. You can say:
+
+    vroom --vroom --skip=50
+
+and it will start on slide #51.
+
+=item vroom <action> --input=<file_name>
+
+This option lets you specify an alternate input file name, instead of the
+default one, C<slides.vroom>.
 
 =back
 
@@ -557,17 +802,23 @@ Each slide can have one or more configuration options. Options are
 a comma separated list that follow the '----' header for a slide.
 Like this:
 
-    ---- center
-    ---- html
-    ---- perl,i20
     ---- config
+    ---- center
+    ---- perl,i20
+    ---- include file-name
+    ---- replace
     ---- skip
 
 =over
 
-=item skip
+=item config
 
-Ignore the following slide completely.
+The slide is really a yaml configuration. It will not be displayed
+in the presentation, but will tell vroom what to do from that point
+forward.
+
+Usually, a C<config> slide is the first thing in your input file, but
+you can use more than one config slide.
 
 =item center
 
@@ -578,44 +829,81 @@ Center the contents of the slide.
 'i' followed by a number means to indent the contents by the number of
 characters.
 
-=item perl,ruby,python,js,yaml,make,html
+=item i-##
+
+'i' followed by a negative number means to strip that number of leading 
+characters from the contents of the slide.  This can be useful if you need
+to have characters special to Vroom::Vroom at the beginning of your lines,
+for example if the contents of your slide is unified diff output.
+
+=item perl,ruby,python,php,javascript,haskell,actionscript,html,yaml,xml,json,make,shell,diff
 
 Specifies that the slide is one of those syntaxen, and that the
 appropriate file extension will be used, thus causing vim to syntax
 highlight the slide.
 
-=item config
+=item include file-path-name
 
-The slide is really a yaml configuration. It will not be displayed
-in the presentation, but will tell vroom what to do from that point
-forward. You can use more than one config slide in your
-C<slides.vroom> file.
+Replace the line with the contents of the specified file. Useful to
+include long files that would make your slides file unruly.
+
+=item replace
+
+With the C<replace> option, the '+' animations in the slide cause the
+content to replace the previous partial slide, rather than append to it.
+
+=item skip
+
+Ignore the following slide completely.
 
 =back
 
-You can specify the following confguration options in a config slide:
+=head2 CONFIG SLIDE OPTIONS
+
+You can specify the following configuration options in a config slide:
 
 =over
 
-=item title <text>
+=item title: <text>
 
 The title of your presentation.
 
-=item height <number>
+=item height: <number>
 
 The number of lines in the terminal you plan to use when presenting the
 show. Used for centering the content.
 
-=item width <number>
+=item width: <number>
 
 The number of columns in the terminal you plan to use when presenting
 the show. Used for centering the content.
 
-=item list_indent <number>
+=item indent: <number>
+
+All slides will be indented by this number of spaces by default.
+
+=item list_indent: <number>
 
 Auto detect slides that have lists in them, and indent them by the
 specified number of columns.
 
+=item vim: <name>
+
+You can specify the name of the vim executable to use. If you set this to
+C<gvim> special gvim support will be provided.
+
+=item GVim options
+
+The following options are available, if your vim option is set to gvim.
+
+    fuopt: maxhorz,maxvert
+    guioptions: egmLtT
+    guicursor: a:blinkon0-ver25-Cursor
+    guifont: Bitstream_Vera_Sans_Mono:h18
+
+These are all documented by gvim's help system. Please see that for more
+information.
+
 =back
 
 =head1 KEY MAPPINGS
@@ -632,15 +920,39 @@ Advance one slide.
 
 Go back one slide.
 
-=item <R>
+=item ??
 
-Run current slide as Perl.
-Turn a YAML slide into Data::Dumper Perl.
+Bring up the help screen.
 
-=item <Q>
+=item RR (or R -- deprecated)
+
+If the current slide is declared Perl, Python, Ruby, PHP, Haskell or
+JavaScript, then run it accordingly.
+
+=item QQ
 
 Quit Vroom.
 
+=item VV
+
+Since these vim options apply while editing the C<slides.vroom> file
+(yes, beware), you can use this shortcut to launch Vroom on the current
+contents whilst writing your slides.
+
+=item EE
+
+Edit the file that the cursor is on the filename of.
+
+You can put file path names in your slides, and then easily bring them
+up during your presentation.
+
+=item OO
+
+On a Mac, run the OS X C<open> command on the argument that your cursor is on.
+
+For instance, if you want to display an image, you could put the file
+path of the image in your slide, then use OO to launch it.
+
 =back
 
 =head1 CUSTOM CONFIGURATION
@@ -652,9 +964,62 @@ it creates.
 Use this file to specify your own custom vim settings for all your vroom
 presentations.
 
+You can also create a file called C<.vroom/gvimrc> for gvim overrides,
+if you are using gvim.
+
+=head1 USING MacVim OR gvim
+
+If you have a Mac, you really should try using MacVim for Vroom slide
+shows. You can run it in fullscreen mode, and it looks kinda
+professional.
+
+To do this, set the vim option in your config section:
+
+    vim: gvim
+
+NOTE: On my Mac, I have gvim symlinked to mvim, which is a smart startup
+      script that ships with MacVim. Ping me, if you have questions
+      about this setup.
+
+=head1 GITHUB
+
+I(ngy) put all my public talks on github. I think it is an excellent way
+to publish your slides and give people a url to review them. Here are
+the things I do to make this work well:
+
+1) I create a repository for every presentation I give. The name of
+   the repo is of the form <topic>-<event/time>-talk. You can go to
+   L<http://github.com/ingydotnet/> and look for the repos ending
+   with '-talk'.
+
+2) GitHub has a feature called gh-pages that you can use to create a
+   website for each github repo. I use this feature to publish the html
+   output of my talk. I do something like this:
+
+    vroom --html
+    mv html /tmp
+    git branch gh-pages
+    git checkout gh-pages
+    rm -r *
+    mv /tmp/html/* .
+    rmdir /tmp/html
+    git add .
+    git commit -m 'Publish my slides'
+    git push
+    git checkout master
+
+3) If my repo is called C<vroom-yapcna2009-talk>, then after I publish
+   the talk to the gh-pages branch, it will be available as
+   L<http://ingydotnet.github.com/vroom-yapcna2009-talk>.
+   I then link this url from
+   L<http://github.com/ingydotnet/vroom-yapcna2009-talk> as the Homepage.
+
+You can see an example of a talk published to HTML and posted via gh-pages
+at L<http://ingydotnet.github.com/vroom-pm/>.
+
 =head1 NOTE
 
-Vroom is called Vroom but the module is Vroom::Vroom because the
+Vroom is called Vroom, but the module is Vroom::Vroom because the
 CPAN shell sometimes thinks Vroom is Tim Vroom, and it refuses to
 install him.
 
